@@ -2113,7 +2113,7 @@ func main() {
 		switch args[0] {
 		case "-version":
 			fmt.Printf("%s\n", eutils.EDirectVersion)
-		case "-help", "help":
+		case "-help", "help", "--help":
 			eutils.PrintHelp("rchive", "rchive-help.txt")
 		case "-extras", "-extra", "-advanced":
 			eutils.PrintHelp("rchive", "rchive-extras.txt")
