@@ -363,7 +363,7 @@ func main() {
 	switch args[0] {
 	case "-version":
 		fmt.Printf("%s\n", eutils.EDirectVersion)
-	case "-help", "help":
+	case "-help", "help", "--help":
 		eutils.PrintHelp("xtract", "xtract-help.txt")
 	case "-examples", "-example":
 		eutils.PrintHelp("xtract", "xtract-examples.txt")
