@@ -33,6 +33,7 @@ package eutils
 import (
 	"encoding/hex"
 	"html"
+	"sort"
 	"strconv"
 	"strings"
 	"sync"
@@ -1303,6 +1304,173 @@ var ncbi4naToIupac = map[int]string{
 	255: "NN",
 }
 
+// use RelaxString to convert non-alphanumeric characters to spaces
+
+// CleanAuthor is used for citation matching
+func CleanAuthor(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = FixMisusedLetters(str, true, true, false)
+	str = TransformAccents(str, false, false)
+
+	// convert numeric encoding to apostrophe
+	str = strings.Replace(str, "&#39;", "'", -1)
+	// remove space following apostrophe
+	str = strings.Replace(str, "' ", "'", -1)
+	// then remove apostrophe to match indexing logic
+	str = strings.Replace(str, "'", "", -1)
+
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleBracket(str) {
+		str = RepairTableMarkup(str, SPACE)
+		// if wrp // str = encodeAngleBracketsAndAmpersand(str)
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	str = strings.Replace(str, "(", " ", -1)
+	str = strings.Replace(str, ")", " ", -1)
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	// remove jr and sr suffix
+	if strings.Index(str, " ") != strings.LastIndex(str, " ") {
+		str = strings.TrimSuffix(str, " jr")
+		str = strings.TrimSuffix(str, " Jr")
+		str = strings.TrimSuffix(str, " sr")
+		str = strings.TrimSuffix(str, " Sr")
+	}
+
+	// normalize remaining punctuation
+	str = GenBankToMedlineAuthors(str)
+
+	// truncate to single initial
+	pos := strings.LastIndex(str, " ")
+	if pos > 0 && len(str) > pos+2 {
+		str = str[:pos+2]
+	}
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// CleanJournal is used for citation matching
+func CleanJournal(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	// convert numeric encoding to apostrophe
+	str = strings.Replace(str, "&#39;", "'", -1)
+	// remove apostrophe, but do not remove trailing space
+	str = strings.Replace(str, "'", "", -1)
+
+	// convert period to space
+	str = strings.Replace(str, ".", " ", -1)
+
+	// convert " & " to " and "
+	str = strings.Replace(str, " &amp; ", " and ", -1)
+
+	words := strings.FieldsFunc(str, func(c rune) bool {
+		// split at non-alphanumeric characters
+		return (!unicode.IsLetter(c) && !unicode.IsDigit(c)) || c > 127
+	})
+	str = strings.Join(words, " ")
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// CleanPage is used for citation matching
+func CleanPage(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	// keep starting page before hyphen
+	str, _ = SplitInTwoLeft(str, "-")
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// CleanTitle is used for citation matching
+func CleanTitle(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = FixMisusedLetters(str, true, false, true)
+	str = TransformAccents(str, false, false)
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleBracket(str) {
+		str = RepairTableMarkup(str, SPACE)
+		// if wrp // str = encodeAngleBracketsAndAmpersand(str)
+	}
+	if HasAmpOrNotASCII(str) {
+		str = html.UnescapeString(str)
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	str = strings.Replace(str, "(", " ", -1)
+	str = strings.Replace(str, ")", " ", -1)
+
+	str = strings.Replace(str, "_", " ", -1)
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// RelaxString removes all punctuation from patterns and search text
+func RelaxString(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	terms := strings.FieldsFunc(str, func(c rune) bool {
+		// split at non-alphanumeric characters
+		return (!unicode.IsLetter(c) && !unicode.IsDigit(c)) || c > 127
+	})
+
+	str = strings.Join(terms, " ")
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
 // CleanupBadSpaces converts non-ASCII Unicode spaces to ASCII space
 func CleanupBadSpaces(str string) string {
 
@@ -1487,6 +1655,8 @@ func CleanupQuery(str string, exactMatch, removeBrackets bool) string {
 // CompressRunsOfSpaces turns runs of spaces into a single space
 func CompressRunsOfSpaces(str string) string {
 
+	// return strings.Join(strings.Fields(str), " ")
+
 	whiteSpace := false
 	var buffer strings.Builder
 
@@ -1835,6 +2005,18 @@ func HasBadSpace(str string) bool {
 	return false
 }
 
+// HasCommaOrSemicolon reports on comma, semicolon, or hyphen
+func HasCommaOrSemicolon(str string) bool {
+
+	for _, ch := range str {
+		if ch == ',' || ch == ';' || ch == '-' {
+			return true
+		}
+	}
+
+	return false
+}
+
 // HasExtraSpaces determines whether RemoveExtraSpaces needs to be called
 func HasExtraSpaces(str string) bool {
 
@@ -1896,6 +2078,18 @@ func HasHyphenOrApostrophe(str string) bool {
 	return false
 }
 
+// HasSpaceOrHyphen checks for multiple words
+func HasSpaceOrHyphen(str string) bool {
+
+	for _, ch := range str {
+		if ch == ' ' || ch == '-' {
+			return true
+		}
+	}
+
+	return false
+}
+
 // HasUnicodeMarkup checks for Unicode superscript or subscript characters
 func HasUnicodeMarkup(str string) bool {
 
@@ -1976,6 +2170,18 @@ func HTMLRepair(str string) (string, bool) {
 	return res, ok
 }
 
+// IsAllCapsOrDigits matches upper-case or digits
+func IsAllCapsOrDigits(str string) bool {
+
+	for _, ch := range str {
+		if !unicode.IsUpper(ch) && !unicode.IsDigit(ch) {
+			return false
+		}
+	}
+
+	return true
+}
+
 // IsAllDigits matches only digits
 func IsAllDigits(str string) bool {
 
@@ -2000,6 +2206,18 @@ func IsAllDigitsOrPeriod(str string) bool {
 	return true
 }
 
+// IsAllLettersOrDigits matches letters or digits
+func IsAllLettersOrDigits(str string) bool {
+
+	for _, ch := range str {
+		if !unicode.IsLetter(ch) && !unicode.IsDigit(ch) {
+			return false
+		}
+	}
+
+	return true
+}
+
 // IsNotASCII returns true for any character greater than 7-bits
 func IsNotASCII(str string) bool {
 
@@ -2135,6 +2353,82 @@ func ParseIndex(indx string) *XMLFind {
 	return &XMLFind{Index: indx, Parent: prnt, Match: match, Attrib: attrib, Versn: versn}
 }
 
+// PrepareForIndexing performs cleanup and normalization of index and query strings
+func PrepareForIndexing(str string, doHomoglyphs, isAuthor, isProse, spellGreek, reEncode bool) string {
+
+	if IsNotASCII(str) {
+		str = FixMisusedLetters(str, doHomoglyphs, isAuthor, isProse)
+		str = TransformAccents(str, spellGreek, reEncode)
+		if HasUnicodeMarkup(str) {
+			str = RepairUnicodeMarkup(str, SPACE)
+		}
+	}
+
+	str = strings.ToLower(str)
+
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAngleBracket(str) {
+		str = RepairEncodedMarkup(str)
+		str = RepairTableMarkup(str, SPACE)
+		str = RepairScriptMarkup(str, SPACE)
+		str = RepairMathMLMarkup(str, SPACE)
+		// RemoveEmbeddedMarkup must be called before UnescapeString, which was suppressed in ExploreElements
+		str = RemoveEmbeddedMarkup(str)
+	}
+
+	if HasAmpOrNotASCII(str) {
+		str = html.UnescapeString(str)
+		str = strings.ToLower(str)
+	}
+
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+
+	str = strings.Replace(str, "(", " ", -1)
+	str = strings.Replace(str, ")", " ", -1)
+
+	str = strings.Replace(str, "_", " ", -1)
+
+	if HasHyphenOrApostrophe(str) {
+		str = FixSpecialCases(str)
+	}
+
+	str = strings.Replace(str, "_", " ", -1)
+	str = strings.Replace(str, "-", " ", -1)
+	str = strings.Replace(str, "+", " ", -1)
+	str = strings.Replace(str, "~", " ", -1)
+
+	// remove trailing punctuation from each word
+	var arry []string
+
+	terms := strings.Fields(str)
+	for _, item := range terms {
+		max := len(item)
+		for max > 1 {
+			ch := item[max-1]
+			if ch != '.' && ch != ',' && ch != ':' && ch != ';' {
+				break
+			}
+			// trim trailing period, comma, colon, and semicolon
+			item = item[:max-1]
+			// continue checking for runs of punctuation at end
+			max--
+		}
+		if item == "" {
+			continue
+		}
+		arry = append(arry, item)
+	}
+
+	// rejoin into string
+	str = strings.Join(arry, " ")
+
+	return str
+}
+
 // RemoveEmbeddedMarkup removes internal mixed-content sections
 func RemoveEmbeddedMarkup(str string) string {
 
@@ -2310,6 +2604,22 @@ func RepairEncodedMarkup(str string) string {
 	return buffer.String()
 }
 
+// RemoveCommaOrSemicolon replaces comma or semicolon with space
+func RemoveCommaOrSemicolon(str string) string {
+
+	str = strings.ToLower(str)
+
+	if HasCommaOrSemicolon(str) {
+		str = strings.Replace(str, ",", " ", -1)
+		str = strings.Replace(str, ";", " ", -1)
+		str = CompressRunsOfSpaces(str)
+	}
+	str = strings.TrimSpace(str)
+	str = strings.TrimRight(str, ".?:")
+
+	return str
+}
+
 // RemoveExtraSpaces removes spaces inside parentheses, after hyphen, and before comma
 func RemoveExtraSpaces(str string) string {
 
@@ -2679,6 +2989,55 @@ func RepairUnicodeMarkup(str string, policy int) string {
 				buffer.WriteRune(ch)
 				continue
 			}
+			switch level {
+			case NOSCRIPT:
+				switch policy {
+				case PERIOD:
+				case SPACE:
+				case CONCISE:
+				case BRACKETS:
+				case MARKDOWN:
+				case SLASH:
+				case TAGS:
+				}
+			case SUPSCRIPT:
+				switch policy {
+				case PERIOD:
+				case SPACE:
+				case CONCISE:
+					buffer.WriteRune(']')
+				case BRACKETS:
+					buffer.WriteRune(']')
+				case MARKDOWN:
+					buffer.WriteRune('^')
+				case SLASH:
+				case TAGS:
+					buffer.WriteString("</sup>")
+				}
+			case SUBSCRIPT:
+				switch policy {
+				case PERIOD:
+				case SPACE:
+				case CONCISE:
+				case BRACKETS:
+					buffer.WriteRune(')')
+				case MARKDOWN:
+					buffer.WriteRune('~')
+				case SLASH:
+				case TAGS:
+					buffer.WriteString("</sub>")
+				}
+			case PLAINDIGIT:
+				switch policy {
+				case PERIOD:
+				case SPACE:
+				case CONCISE:
+				case BRACKETS:
+				case MARKDOWN:
+				case SLASH:
+				case TAGS:
+				}
+			}
 			level = NOSCRIPT
 		} else if ch >= '0' && ch <= '9' {
 			switch level {
@@ -2845,6 +3204,24 @@ func RepairUnicodeMarkup(str string, policy int) string {
 	return buffer.String()
 }
 
+// SortStringByWords sorts the individual words in a string
+func SortStringByWords(str string) string {
+
+	str = RemoveCommaOrSemicolon(str)
+
+	// check for multiple words
+	if HasSpaceOrHyphen(str) {
+		flds := strings.Fields(str)
+		sort.Slice(flds, func(i, j int) bool { return flds[i] < flds[j] })
+		str = strings.Join(flds, " ")
+		str = strings.Replace(str, "-", " ", -1)
+		str = CompressRunsOfSpaces(str)
+		str = strings.TrimRight(str, ".?:")
+	}
+
+	return str
+}
+
 // SplitInTwoLeft loads the first argument if no delimiter is present
 func SplitInTwoLeft(str, chr string) (string, string) {
 
